from enum import Enum
from typing import Union

from meross_iot.cloud.abilities import HUB_MTS100_ALL, HUB_MTS100_TEMPERATURE, HUB_MTS100_MODE, HUB_TOGGLEX, HUB_ONLINE
from meross_iot.cloud.devices.subdevices.generic import GenericSubDevice
from meross_iot.logger import VALVES_LOGGER as l
from meross_iot.meross_event import DeviceSwitchStatusEvent, ThermostatTemperatureChange, ThermostatModeChange, \
    DeviceOnlineStatusEvent


class ThermostatMode(Enum):
    COMFORT = 1
    CUSTOM = 0
    ECONOMY = 2
    SCHEDULE = 3


class ThermostatV3Mode(Enum):
    AUTO = 3
    COOL = 2
    CUSTOM = 0
    ECONOMY = 4
    HEAT = 1


class ValveSubDevice(GenericSubDevice):

    def __init__(self, cloud_client, subdevice_id, parent_hub, **kwords):
        super().__init__(cloud_client, subdevice_id, parent_hub, **kwords)

    def _handle_push_notification(self, namespace, payload, from_myself=False):
        # Let the Generic handler to handle the common events.
        handled = super()._handle_push_notification(namespace=namespace, payload=payload, from_myself=from_myself)
        if handled:
            return True

        # If the parent handler was unable to parse it, we do it here.
        evt = None
        if namespace == HUB_MTS100_ALL:
            self._raw_state.update(payload)
            return True

        elif namespace == HUB_TOGGLEX:
            togglex = self._raw_state.get('togglex')
            if togglex is None:
                togglex = {}
                self._raw_state['togglex'] = togglex
            togglex.update(payload)
            evt = DeviceSwitchStatusEvent(self, 0, self.onoff, from_myself)
            self.fire_event(evt)
            return True

        elif namespace == HUB_MTS100_MODE:
            mode = self._raw_state.get('mode')
            if mode is None:
                mode = {}
                self._raw_state['mode'] = mode

            mode.update(payload)
            evt = ThermostatModeChange(device=self, mode=self.mode, generated_by_myself=from_myself)
            self.fire_event(evt)
            return True

        elif namespace == HUB_MTS100_TEMPERATURE:
            temp = self._raw_state.get('temperature')
            if temp is None:
                temp = {}
                self._raw_state['temperature'] = temp

            temp.update(payload)
            evt = ThermostatTemperatureChange(device=self,
                                              temperature_state=self._raw_state.get('temperature'),
                                              generated_by_myself=from_myself)
            self.fire_event(evt)
            return True

        # TODO: handle TIME SYNC event?
        # elif namespace == HUB_TIME_SYNC:
        #    self._state.get('??').update(payload)

        else:
            l.warn("Unsupported/unhandled event: %s" % namespace)
            l.debug("Namespace: %s, Data: %s" % (namespace, payload))
            return False

    @property
    def onoff(self):
        onoff = self._get_property('togglex', 'onoff')
        return onoff

    @property
    def heating(self):
        heating = self._get_property('temperature', 'heating')
        if heating is None:
            return None
        else:
            return heating == 1

    @property
    def room_temperature(self):
        temp = self._get_property('temperature', 'room')
        if temp is None:
            return None
        else:
            return temp / 10

    @property
    def target_temperature(self):
        """
        Returns the current target temperature
        :return:
        """
        # The API returns the temperature in decimals.
        # For this reason, we convert it to integers.
        temp = self._get_property('temperature', 'currentSet')
        if temp is None:
            return None
        else:
            return temp / 10

    def set_target_temperature(self,
                               target_temp: float = None,
                               callback: callable = None):
        """
        Sets the target temperature of the thermostat
        :param target_temp: temperature to be set
        :param callback:
        :return:
        """
        # The API expects the target temperature in DECIMALS, so we need to multiply the user's input by 10
        value = target_temp * 10
        payload = {'temperature': [{'id': self.subdevice_id, 'custom': value}]}
        return self.execute_command(command='SET',
                                    namespace=HUB_MTS100_TEMPERATURE,
                                    payload=payload,
                                    callback=callback)

    def set_preset_temperature(self,
                               away: float = None,
                               comfort: float = None,
                               economy: float = None,
                               callback: callable = None):
        """
        Configures the preset temperature values. The temperature values should be expressed in
        Celsius degrees.
        :param away: Target temperature for away preset
        :param comfort: Target temperature for comfort preset
        :param economy: Target temperature for economy preset
        :param callback:
        :return:
        """
        # The API expects the celsius degrees in DECIMALS, so we need to multiply the user's input by 10
        temperature_conf = {'id': self.subdevice_id}
        if away is not None:
            temperature_conf['away'] = away * 10
        if comfort is not None:
            temperature_conf['comfort'] = comfort * 10
        if economy is not None:
            temperature_conf['economy'] = economy * 10

        payload = {'temperature': [temperature_conf]}
        return self.execute_command(command='SET',
                                    namespace=HUB_MTS100_TEMPERATURE,
                                    payload=payload,
                                    callback=callback)

    @property
    def mode(self):
        state = self._get_property('mode', 'state')
        if state is None:
            return None
        else:
            # Parse the mode according to the current device type
            if self.type == 'mts100v3':
                return ThermostatV3Mode(state)
            elif self.type == 'mts100':
                return ThermostatMode(state)
            else:
                l.error("The current thermostat mode is not supported.")
                return None

    def set_mode(self, mode: Union[ThermostatV3Mode, ThermostatMode, int]):
        """
        Sets the temperature mode for the thermostat
        :param mode:
        :return:
        """
        # Make sure we are passing correct values
        if self.type == 'mts100v3' and not isinstance(mode, ThermostatV3Mode):
            raise ValueError("This thermostat only supports ThermostatV3Mode modes")
        elif self.type == 'mts100' and not isinstance(mode, ThermostatMode):
            raise ValueError("This thermostat only supports ThermostatMode modes")
        elif isinstance(mode, int):
            l.warning("Setting a raw integer value as mode. This is not recommended. "
                      "Please use ThermostatMode or ThermostatV3Mode")
        self.execute_command('SET', HUB_MTS100_MODE, {'mode': [{'id': self.subdevice_id, 'state': mode.value}]})

    def _togglex(self, onoff, channel=0, callback=None):
        payload = {'togglex': [{'channel': channel, 'id': self.subdevice_id, 'onoff': onoff}]}
        return self.execute_command('SET', HUB_TOGGLEX, payload, callback=callback)

    def turn_on(self, callback=None):
        return self._togglex(onoff=1, callback=callback)  # TODO: test this

    def turn_off(self, callback=None):
        return self._togglex(onoff=0, callback=callback)  # TODO: test this



import copy
import json
import random
import ssl
import string
import time
import uuid as UUID
from hashlib import md5
from threading import Event, RLock

import paho.mqtt.client as mqtt

from meross_iot.cloud.client_status import ClientStatus
from meross_iot.cloud.connection import ConnectionStatusManager
from meross_iot.cloud.exceptions.CommandTimeoutException import CommandTimeoutException
from meross_iot.cloud.timeouts import SHORT_TIMEOUT
from meross_iot.credentials import MerossCloudCreds
from meross_iot.logger import CONNECTION_MANAGER_LOGGER as l
from meross_iot.logger import NETWORK_DATA as networkl
from meross_iot.utilities.synchronization import AtomicCounter


def build_client_request_topic(client_uuid):
    return "/appliance/%s/subscribe" % client_uuid


class PendingMessageResponse(object):
    """
    This class is used as an Handle for mqtt messages that expect an ACK back.
    When a callback is passed to the constructor, this object is configured as an "async" waiter.
    Instead, passing a None callback, makes this object to act as a synchronously waiter.
    It is meant to be used internally by the library, in order to handle ACK waiting and callback calling.
    Note that this object is not thread safe.
    """
    _message_id = None
    _callback = None
    _event = None
    _response = None
    _error = None

    def __init__(self, message_id, callback=None):
        self._message_id = message_id

        # Only instantiate an event if no callback has been specified
        if callback is None:
            self._event = Event()
        else:
            self._callback = callback

    def wait_for_response(self, timeout=SHORT_TIMEOUT):
        """
        This method blocks until an ACK/RESPONSE message is received for the corresponding message_id that it refers
        to. Note that this method only works when the user is synchronously waiting for the response message.
        This method raises an exception if invoked when a callback was specified in the constructor.
        :param timeout:
        :return:
        """
        if self._event is None:
            raise Exception("Error: you can invoke this method only if you don't use a callback (i.e. sync invocation)")

        # Wait until we receive the message.
        # If timeout occurs, return failure and None as received message.
        success = self._event.wait(timeout=timeout)
        return success, self._response

    def notify_message_received(self, error=None, response=None):
        self._response = copy.deepcopy(response)
        self._error = error

        if self._event is not None:
            self._event.set()
        elif self._callback is not None:
            try:
                self._callback(self._error, self._response)
            except:
                l.exception("Unhandled error occurred while executing the callback")


class MerossCloudClient(object):
    # Meross Cloud credentials, which are provided by the HTTP Api.
    _cloud_creds = None

    # Connection info
    connection_status = None
    _domain = None
    _port = 2001
    _ca_cert = None

    # App and client ID
    _app_id = None
    _client_id = None

    # Paho mqtt client object
    _mqtt_client = None

    # Callback to be invoked every time a push notification is received from the MQTT broker
    _push_message_callback = None

    # This dictionary is used to keep track of messages issued to the broker that are waiting for an ACK
    # The key is the message_id, the value is the PendingMessageResponse object.
    # Access to this resource is protected with exclusive locking
    _pending_response_messages = None
    _pending_responses_lock = None

    def __init__(self,
                 cloud_credentials,             # type: MerossCloudCreds
                 push_message_callback=None,    # type: callable
                 **kwords):

        self.connection_status = ConnectionStatusManager()
        self._cloud_creds = cloud_credentials
        self._pending_response_messages = dict()
        self._pending_responses_lock = RLock()
        self._push_message_callback = push_message_callback
        self._subscription_count = AtomicCounter(0)

        if "domain" in kwords:
            self._domain = kwords['domain']
        else:
            self._domain = "iot.meross.com"

        # Lookup port and certificate for MQTT server
        self._port = kwords.get('port', MerossCloudClient._port)
        self._ca_cert = kwords.get('ca_cert', None)

        self._generate_client_and_app_id()

        # Password is calculated as the MD5 of USERID concatenated with KEY
        md5_hash = md5()
        clearpwd = "%s%s" % (self._cloud_creds.user_id, self._cloud_creds.key)
        md5_hash.update(clearpwd.encode("utf8"))
        hashed_password = md5_hash.hexdigest()

        # Start the mqtt client
        self._mqtt_client = mqtt.Client(client_id=self._client_id,
                                        protocol=mqtt.MQTTv311)  # ex. app-id -> app:08d4c9f99da40203ebc798a76512ec14
        self._mqtt_client.on_connect = self._on_connect
        self._mqtt_client.on_message = self._on_message
        self._mqtt_client.on_disconnect = self._on_disconnect
        self._mqtt_client.on_subscribe = self._on_subscribe

        # Avoid login if user_id is None
        if self._cloud_creds.user_id is not None:
            self._mqtt_client.username_pw_set(username=self._cloud_creds.user_id,
                                              password=hashed_password)
        self._mqtt_client.tls_set(ca_certs=self._ca_cert, certfile=None,
                                  keyfile=None, cert_reqs=ssl.CERT_REQUIRED,
                                  tls_version=ssl.PROTOCOL_TLS,
                                  ciphers=None)

    def close(self):
        l.info("Closing the MQTT connection...")
        self._mqtt_client.disconnect()
        l.debug("Waiting for the client to disconnect...")
        self.connection_status.wait_for_status(ClientStatus.CONNECTION_DROPPED)

        # Starts a new thread that handles mqtt protocol and calls us back via callbacks
        l.debug("Stopping the MQTT looper.")
        self._mqtt_client.loop_stop(True)

        l.info("Client has been fully disconnected.")

    def connect(self):
        """
        Starts the connection to the MQTT broker
        :return:
        """
        l.info("Initializing the MQTT connection...")
        self._mqtt_client.connect(self._domain, self._port, keepalive=30)
        self.connection_status.update_status(ClientStatus.CONNECTING)

        # Starts a new thread that handles mqtt protocol and calls us back via callbacks
        l.debug("(Re)Starting the MQTT looper.")
        self._mqtt_client.loop_stop(True)
        self._mqtt_client.loop_start()

        l.debug("Waiting for the client to connect...")
        self.connection_status.wait_for_status(ClientStatus.SUBSCRIBED)
        l.info("Client connected to MQTT broker and subscribed to relevant topics.")

    # ------------------------------------------------------------------------------------------------
    # MQTT Handlers
    # ------------------------------------------------------------------------------------------------
    def _on_disconnect(self, client, userdata, rc):
        l.info("Disconnection detected. Reason: %s" % str(rc))

        # When the mqtt connection is dropped, we need to reset the subscription counter.
        self._subscription_count = AtomicCounter(0)
        self.connection_status.update_status(ClientStatus.CONNECTION_DROPPED)

        # TODO: should we handle disconnection in some way at this level?

        if rc == mqtt.MQTT_ERR_SUCCESS:
            pass
        else:
            client.loop_stop(True)

    def _on_unsubscribe(self):
        l.debug("Unsubscribed from topic")
        self._subscription_count.dec()

    def _on_subscribe(self, client, userdata, mid, granted_qos):
        l.debug("Succesfully subscribed to topic. Subscription count: %d" % self._subscription_count.get())
        if self._subscription_count.inc() == 2:
            self.connection_status.update_status(ClientStatus.SUBSCRIBED)

    def _on_connect(self, client, userdata, rc, other):
        l.debug("Connected with result code %s" % str(rc))
        self.connection_status.update_status(ClientStatus.CONNECTED)

        self._client_response_topic = "/app/%s-%s/subscribe" % (self._cloud_creds.user_id, self._app_id)
        self._user_topic = "/app/%s/subscribe" % self._cloud_creds.user_id

        # Subscribe to the relevant topics
        l.debug("Subscribing to topics...")
        client.subscribe(self._user_topic)
        client.subscribe(self._client_response_topic)

    def _on_message(self, client, userdata, msg):
        """
        This handler is called when a message is received from the MQTT broker, on the subscribed topics.
        The current implementation checks the validity of the message itself, by verifying its signature.

        :param client: is the MQTT client reference, useful to respond back
        :param userdata: metadata about the received data
        :param msg: message that was received
        :return: nothing, it simply handles the message accordingly.
        """
        networkl.debug(msg.topic + " --> " + str(msg.payload))

        try:
            message = json.loads(str(msg.payload, "utf8"))
            header = message['header']

            message_hash = md5()
            strtohash = "%s%s%s" % (header['messageId'], self._cloud_creds.key, header['timestamp'])
            message_hash.update(strtohash.encode("utf8"))
            expected_signature = message_hash.hexdigest().lower()

            if header['sign'] != expected_signature:
                # TODO: custom exception for invalid signature
                raise Exception('The signature did not match!')

            # Check if there is any thread waiting for this message or if there is a callback that we need to invoke.
            # If so, do it here.
            handle = None
            with self._pending_responses_lock:
                msg_id = header['messageId']
                handle = self._pending_response_messages.get(msg_id)

            from_myself = False
            if handle is not None:
                # There was a handle for this message-id. It means it is a response message to some
                # request performed by the library itself.
                from_myself = True
                try:
                    l.debug("Calling handle event handler for message %s" % msg_id)
                    # Call the handler
                    handle.notify_message_received(error=None, response=message)
                    l.debug("Done handler for message %s" % msg_id)

                    # Remove the message from the pending queue
                    with self._pending_responses_lock:
                        del self._pending_response_messages[msg_id]
                except:
                    l.exception("Error occurred while invoking message handler")

            # Let's also catch all the "PUSH" notifications and dispatch them to the push_notification_callback.
            if self._push_message_callback is not None and header['method'] == "PUSH" and 'namespace' in header:
                self._push_message_callback(message, from_myself=from_myself)

        except Exception:
            l.exception("Failed to process message.")

    # ------------------------------------------------------------------------------------------------
    # Protocol Handlers
    # ------------------------------------------------------------------------------------------------
    def execute_cmd(self, dst_dev_uuid, method, namespace, payload, callback=None, timeout=SHORT_TIMEOUT):
        start = time.time()
        # Build the mqtt message we will send to the broker
        message, message_id = self._build_mqtt_message(method, namespace, payload)

        # Register the waiting handler for that message
        handle = PendingMessageResponse(message_id=message_id, callback=callback)
        with self._pending_responses_lock:
            self._pending_response_messages[message_id] = handle

        # Send the message to the broker
        l.debug("Executing message-id %s, %s on %s command for device %s" % (message_id, method,
                                                                             namespace, dst_dev_uuid))
        self._mqtt_client.publish(topic=build_client_request_topic(dst_dev_uuid), payload=message)

        # If the caller has specified a callback, we don't need to actively wait for the message ACK. So we can
        # immediately return.
        if callback is not None:
            return None

        # Otherwise, we need to wait until the message is received.
        l.debug("Waiting for response to message-id %s" % message_id)
        success, resp = handle.wait_for_response(timeout=timeout)
        if not success:
            raise CommandTimeoutException("A timeout occurred while waiting for the ACK: %d" % timeout)

        elapsed = time.time() - start

        l.debug("Message-id: %s, command %s-%s command for device %s took %s" % (message_id, method,
                                                                                 namespace, dst_dev_uuid, str(elapsed)))
        return resp['payload']

    # ------------------------------------------------------------------------------------------------
    # Protocol utilities
    # ------------------------------------------------------------------------------------------------
    def _build_mqtt_message(self, method, namespace, payload):
        """
        Sends a message to the Meross MQTT broker, respecting the protocol payload.
        :param method:
        :param namespace:
        :param payload:
        :return:
        """

        # Generate a random 16 byte string
        randomstring = ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(16))

        # Hash it as md5
        md5_hash = md5()
        md5_hash.update(randomstring.encode('utf8'))
        messageId = md5_hash.hexdigest().lower()
        timestamp = int(round(time.time()))

        # Hash the messageId, the key and the timestamp
        md5_hash = md5()
        strtohash = "%s%s%s" % (messageId, self._cloud_creds.key, timestamp)
        md5_hash.update(strtohash.encode("utf8"))
        signature = md5_hash.hexdigest().lower()

        data = {
            "header":
                {
                    "from": self._client_response_topic,
                    "messageId": messageId,  # Example: "122e3e47835fefcd8aaf22d13ce21859"
                    "method": method,  # Example: "GET",
                    "namespace": namespace,  # Example: "Appliance.System.All",
                    "payloadVersion": 1,
                    "sign": signature,  # Example: "b4236ac6fb399e70c3d61e98fcb68b74",
                    "timestamp": timestamp
                },
            "payload": payload
        }
        strdata = json.dumps(data)
        return strdata.encode("utf-8"), messageId

    def _generate_client_and_app_id(self):
        md5_hash = md5()
        rnd_uuid = UUID.uuid4()
        md5_hash.update(("%s%s" % ("API", rnd_uuid)).encode("utf8"))
        self._app_id = md5_hash.hexdigest()
        self._client_id = 'app:%s' % md5_hash.hexdigest()



import json
import logging
import ssl
import uuid as UUID
from hashlib import md5
from os import path, environ
from threading import Event
from zipfile import ZipFile

import paho.mqtt.client as mqtt

from meross_iot.cloud.client import build_client_request_topic
from meross_iot.cloud.device import AbstractMerossDevice
from meross_iot.logger import h, ROOT_MEROSS_LOGGER
from meross_iot.manager import MerossHttpClient
from meross_iot.manager import MerossManager

SNIFF_LOG_FILE = 'sniff.log'
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
l = logging.getLogger().getChild("Sniffer")
l.setLevel(logging.DEBUG)
lhandler = logging.FileHandler(mode='w', filename=SNIFF_LOG_FILE)
lhandler.setFormatter(formatter)
l.addHandler(lhandler)


ROOT_LOG_FILE = 'root.log'
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
r = logging.getLogger()
r.setLevel(logging.DEBUG)
rhandler = logging.FileHandler(mode='w', filename=ROOT_LOG_FILE)
rhandler.setFormatter(formatter)
r.addHandler(rhandler)
r.setLevel(logging.DEBUG)

ROOT_MEROSS_LOGGER.removeHandler(h)


class DeviceSniffer(object):
    def __init__(self, user_id, hashed_password, target_device_uuid, ca_cert=None, domain="iot.meross.com", port=2001):
        self.connect_event = Event()
        self.subscribe_event = Event()

        self.domain = domain
        self.port = port
        self.target_device_uuid = target_device_uuid

        # Generate random app and client id
        md5_hash = md5()
        rnd_uuid = UUID.uuid4()
        md5_hash.update(("%s%s" % ("API", rnd_uuid)).encode("utf8"))
        self._app_id = md5_hash.hexdigest()
        self._client_id = 'app:%s' % md5_hash.hexdigest()

        self._mqtt_client = mqtt.Client(client_id=self._client_id,
                                        protocol=mqtt.MQTTv311)

        self._mqtt_client.on_connect = self._on_connect
        self._mqtt_client.on_message = self._on_message
        self._mqtt_client.on_disconnect = self._on_disconnect
        self._mqtt_client.on_subscribe = self._on_subscribe

        # Avoid login if user_id is None
        if user_id is not None:
            self._mqtt_client.username_pw_set(username=user_id,
                                              password=hashed_password)
        self._mqtt_client.tls_set(ca_certs=ca_cert, certfile=None,
                                  keyfile=None, cert_reqs=ssl.CERT_REQUIRED,
                                  tls_version=ssl.PROTOCOL_TLS,
                                  ciphers=None)

    def _on_connect(self, client, userdata, rc, other):
        l.debug("Connected to MQTT Broker")
        self.connect_event.set()

    def start(self):
        """
        Starts the connection to the MQTT broker
        :return:
        """
        l.debug("Initializing the MQTT connection...")
        self._mqtt_client.connect(self.domain, self.port, keepalive=30)

        # Starts a new thread that handles mqtt protocol and calls us back via callbacks
        l.debug("(Re)Starting the MQTT loop.")
        self._mqtt_client.loop_stop(True)
        self._mqtt_client.loop_start()
        self.connect_event.wait()

        # Subscribe to the corresponding topics ...
        topic = build_client_request_topic(self.target_device_uuid)
        self._mqtt_client.subscribe(topic)
        self.subscribe_event.wait()

    def stop(self):
        self._mqtt_client.disconnect()

    def _on_subscribe(self, client, userdata, mid, granted_qos):
        l.debug("Subscribed correctly")
        self.subscribe_event.set()

    def _on_message(self, client, userdata, msg):
        message = json.loads(str(msg.payload, "utf8"))
        header = message['header']
        l.info(message)

    def _on_disconnect(self, client, userdata, rc):
        l.debug("Disconnected from MQTT brocker")


def main():
    print("Welcome to the Sniffer utility. This python script will gather some useful information about your "
          "Meross devices. All the collected information will be zipped into a zip archive. "
          "You could share such zip file with the developers to help them add support for your device. "
          "Although this utility won't collect your email/password, we recommend you to change "
          "your Meross account password to a temporary one before using this software. Once you are done, "
          "you can restore back your original password. By doing so, you are 100% sure you are not leaking any "
          "real password to the developers.")
    email = environ.get("MEROSS_EMAIL")
    password = environ.get("MEROSS_PASSWORD")

    if email is None or password is None:
        email = input("Please specify your meross email: ")
        email = email.strip()
        password = input("Please specify your meross password: ")
        password = password.strip()

    devices = []
    http = None
    try:
        http = MerossHttpClient(email, password)
        print("# Collecting devices via HTTP api...")
        devices = http.list_devices()
        l.info("DEVICE LISTING VIA HTTP: %s" % devices)
    except:
        print("An error occurred while retrieving Meross devices.")
        exit(1)

    # If the login was successful, start the Meross Manager to "log" data
    manager = MerossManager(meross_email=email, meross_password=password)
    manager.start()

    print("# Listing ONLINE devices...")
    device_registry = {}
    for i, dev in enumerate(devices):
        if dev.get('onlineStatus') != 1:
            continue
        device_registry[str(i)] = dev
        print(f"{i}: {dev.get('devName')} ({dev.get('deviceType')})")

    if len(device_registry.keys())<1:
        print("No devices found or all devices are offline.")
        exit(0)

    while True:
        selection = input("Select the device you want to study: ")
        selection = selection.strip()
        selected_device = device_registry.get(selection)
        if selected_device is not None:
            break

    print("")
    print(f"You have selected {selected_device.get('devName')}.")

    # Start the manager
    creds = http.get_cloud_credentials()
    md5_hash = md5()
    clearpwd = "%s%s" % (creds.user_id, creds.key)
    md5_hash.update(clearpwd.encode("utf8"))
    hashed_password = md5_hash.hexdigest()
    sniffer = DeviceSniffer(
        creds.user_id,
        hashed_password,
        selected_device.get('uuid'),
        ca_cert=None,
        domain=selected_device.get('domain') or "iot.meross.com"
    )

    print("Starting the sniffer...")
    sniffer.start()
    print("You can now start commanding this device from the Meross APP. This utility will catch "
          "every command you send from the app to this device. Such data could help developers to "
          "reproduce the functionality on this library. When DONE, press ENTER to finish.")

    input("Press ENTER to finish.")
    sniffer.stop()

    # As very last step, try to collect data via get_all() and get_abilities
    print("Collecting state info...")
    try:
        d = manager.get_device_by_uuid(selected_device.get('uuid')) # type: AbstractMerossDevice
        sysdata = d.get_sys_data()
        abilities = d.get_abilities()
        l.info(f"Sysdata for {d.uuid}: {sysdata}")
        l.info(f"Abilities for {d.uuid}: {abilities}")
    except:
        l.exception(f"Could not collect sysdata/abilities for {selected_device.get('uuid')}")

    manager.stop()

    print("Collecting logs...")
    zipObj = ZipFile('data.zip', 'w')
    zipObj.write(SNIFF_LOG_FILE)
    zipObj.write(ROOT_LOG_FILE)
    zipObj.close()

    print("A zipfile has been created containing the logs collected during this execution. "
          "It is located in {path}.".format(path=path.abspath(zipObj.filename)))
    print("Thanks for helping the Meross community!")


if __name__ == '__main__':
    main()




import logging
from os import path
from zipfile import ZipFile

from meross_iot.cloud.device import AbstractMerossDevice
from meross_iot.cloud.devices.hubs import GenericHub
from meross_iot.logger import h
from meross_iot.manager import MerossManager

ROOT_LOG_FILE = 'root.log'
INFO_LOG_FILE = 'info.log'

formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

root = logging.getLogger()
root.setLevel(logging.DEBUG)
rhandler = logging.FileHandler(mode='w', filename=ROOT_LOG_FILE)
rhandler.setFormatter(formatter)
rhandler.setLevel(logging.DEBUG)
root.addHandler(rhandler)

l = root.getChild("InfoGather")
l.setLevel(logging.DEBUG)
lhandler = logging.FileHandler(mode='w', filename=INFO_LOG_FILE)
lhandler.setFormatter(formatter)
l.addHandler(lhandler)
h.setLevel(logging.ERROR)


def describe_device(
        d,  # type: AbstractMerossDevice
        manager
                    ):

    l.info("Found device {name}, type: {type}, uuid: {uuid}, firmware: {firmware}, hardware: {hardware}".format(
           name=d.name, type=d.type, uuid=d.uuid, firmware=d.fwversion, hardware=d.hwversion))

    # Gathering status
    sysdata = d.get_sys_data()
    l.info("GET ALL on {uuid} = {sysdata}".format(uuid=d.uuid, sysdata=sysdata))

    # Gathering abilities
    abilities = d.get_abilities()
    l.info("GET ABILITIES on {uuid} = {abilities}".format(uuid=d.uuid, abilities=abilities))

    if isinstance(d, GenericHub):
        subdevices = manager._http_client.list_hub_subdevices(d.uuid)
        l.info("SUBDEVICES of hub {uuid} = {subdevices}".format(uuid=d.uuid, subdevices=subdevices))


def main():
    print("Welcome to the InfoGather software. This python script will gather some useful information about your "
          "Meross devices. All the collected information will be zipped into a zip archive. "
          "You could share such zip file with the developers to help them add support for your device. "
          "Although this utility won't collect your email/password, we recommend you to change "
          "your Meross account password to a temporary one before using this software. Once you are done, "
          "you can restore back your original password. By doing so, you are 100% sure you are not leaking any "
          "real password to the developers.")
    email = input("Please specify your meross email: ")
    email = email.strip()
    password = input("Please specify your meross password: ")
    password = password.strip()
    devices = []
    manager = None

    try:
        manager = MerossManager(meross_email=email, meross_password=password)
        client = manager._http_client
        print("# Collecting devices via HTTP api...")

        devices = client.list_devices()
        l.info("DEVICE LISTING VIA HTTP: %s" % devices)
    except:
        print("An error occurred while retrieving Meross devices.")
        exit(1)

    print("# Starting manager...")
    manager.start()

    print("# Gathering info about supported devices...")
    devices = manager.get_supported_devices()
    for d in devices:
        describe_device(d, manager)

    print("# OK. You can now play with the Meross official APP with your devices to trigger some actions. Hopefully"
          " this program will be able to collect necessary data to help the developers implement its support.")
    print("Once done, simply press ENTER and the program will stop gathering information and return.")
    input("Press ENTER to stop recording data and exit")

    print("Collecting logs...")
    zipObj = ZipFile('data.zip', 'w')
    zipObj.write(ROOT_LOG_FILE)
    zipObj.write(INFO_LOG_FILE)
    zipObj.close()

    print("A zipfile has been created containing the logs collected during this execution. "
          "It is located in {path}.".format(path=path.abspath(zipObj.filename)))
    print("Thanks for helping the Meross community!")


if __name__ == '__main__':
    main()




import logging
from logging import StreamHandler
from sys import stdout

ROOT_MEROSS_LOGGER = logging.getLogger("meross")
API_LOGGER = ROOT_MEROSS_LOGGER.getChild("api")
MANAGER_LOGGER = ROOT_MEROSS_LOGGER.getChild("manager")
CONNECTION_MANAGER_LOGGER = ROOT_MEROSS_LOGGER.getChild("connection")
NETWORK_DATA = ROOT_MEROSS_LOGGER.getChild("network_data")
POWER_PLUGS_LOGGER = ROOT_MEROSS_LOGGER.getChild("power_plugs")
HUB_LOGGER = ROOT_MEROSS_LOGGER.getChild("hub")
BULBS_LOGGER = ROOT_MEROSS_LOGGER.getChild("light_bulbs")
SUBDEVICE_LOGGER = ROOT_MEROSS_LOGGER.getChild("generic_subdevice")
VALVES_LOGGER = ROOT_MEROSS_LOGGER.getChild("valves")
DEVICE_LOGGER = ROOT_MEROSS_LOGGER.getChild("generic_device")


h = StreamHandler(stream=stdout)
ROOT_MEROSS_LOGGER.addHandler(h)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
h.setFormatter(formatter)


# Call this module to adjust the verbosity of the stream output. By default, only INFO is written to STDOUT log.
def set_log_level(root=logging.DEBUG, connection=logging.INFO, network=logging.INFO):
    ROOT_MEROSS_LOGGER.setLevel(root)
    CONNECTION_MANAGER_LOGGER.setLevel(connection)
    NETWORK_DATA.setLevel(network)
